﻿namespace NetinaShop.Repository.Handlers.Products;

public class GetProductsQueryHandler : IRequestHandler<GetProductsQuery, List<ProductSDto>>
{
    private readonly IRepositoryWrapper _repositoryWrapper;

    public GetProductsQueryHandler(IRepositoryWrapper repositoryWrapper)
    {
        _repositoryWrapper = repositoryWrapper;
    }
    public async Task<List<ProductSDto>> Handle(GetProductsQuery request, CancellationToken cancellationToken)
    {
        var products = _repositoryWrapper.SetRepository<Product>().TableNoTracking;
        if (request.CategoryId != default)
            products = products.Where(p => p.CategoryId == request.CategoryId);
        if (request.BrandId != default)
            products = products.Where(p => p.BrandId == request.BrandId);
        if (request.MinPrice > -1)
            products = products.Where(p => p.Cost >= request.MinPrice);
        if (request.MaxPrice > 0)
            products = products.Where(p => p.Cost <= request.MaxPrice);
        if (request.SortBy != QuerySortBy.None)
        {
            switch (request.SortBy)
            {
                case QuerySortBy.Cheapest:
                    products = products.OrderBy(p => p.Cost);
                    break;
                case QuerySortBy.MostExpensive:
                    products = products.OrderByDescending(p => p.Cost);
                    break;
                case QuerySortBy.MostPopular:
                    products = products.OrderByDescending(p => p.Rate);
                    break;
                case QuerySortBy.MostViewed:
                    products = products.OrderByDescending(p => p.Viewed);
                    break;
            }
        }

        return await products.Skip(request.Page * 10)
            .Take(10)
            .Select(ProductMapper.ProjectToSDto)
            .ToListAsync(cancellationToken);
    }
}